// db2eqstartMain.c
//
// main file for db2eqstart
//
// This wizard-generated code is based on code adapted from the
// stationery files distributed as part of the Palm OS SDK 4.0.
//
// Copyright (c) 1999-2000 Palm, Inc. or its subsidiaries.
// All rights reserved.

#include <PalmOS.h>

#include "db2eqstart.h"
#include "db2eqstartRsc.h"
#include "isyncore.h"

// ********************************************************************
// Entry Points
// ********************************************************************

// ********************************************************************
// Global variables
// ********************************************************************

// g_prefs
// cache for application preferences during program execution
db2eqstartPreferenceType g_prefs;

// ********************************************************************
// Internal Constants
// ********************************************************************

// Define the minimum OS version we support
#define ourMinVersion    sysMakeROMVersion(3,0,0,sysROMStageDevelopment,0)
#define kPalmOS10Version sysMakeROMVersion(1,0,0,sysROMStageRelease,0)

HISCSERV	hSyncServ = NULL;
HISCCONF	hSyncConf = NULL;
HISCENG		hSyncer = NULL;
	
static void getFieldText(FieldPtr fld, char *textBuf, int bufSize);
static void updateFieldText(FieldPtr fld, char *fieldText);
int      syncStartup();
void     syncShutdown();



// ********************************************************************
// Internal Functions
// ********************************************************************

// FUNCTION: GetObjectPtr
//
// DESCRIPTION:
//
// This routine returns a pointer to an object in the current form.
//
// PARAMETERS:
//
// formId
//     id of the form to display
//
// RETURNED:
//     address of object as a void pointer

static void * GetObjectPtr(UInt16 objectID)
{
    FormType * frmP;

    frmP = FrmGetActiveForm();
    return FrmGetObjectPtr(frmP, FrmGetObjectIndex(frmP, objectID));
}

// FUNCTION: MainFormInit
//
// DESCRIPTION: This routine initializes the MainForm form.
//
// PARAMETERS:
//
// frm
//     pointer to the MainForm form.

static void MainFormInit(FormType * /*frmP*/)
{
}

// FUNCTION: MainFormDoCommand
//
// DESCRIPTION: This routine performs the menu command specified.
//
// PARAMETERS:
//
// command
//     menu item id

static Boolean MainFormDoCommand(UInt16 command)
{
    Boolean handled = false;
    FormType * frmP;    

    switch (command)
    {
    case OptionsAboutdb2eqstart:

        // Clear the menu status from the display
        MenuEraseStatus(0);

        // Display the About Box.
        frmP = FrmInitForm (AboutForm);
        FrmDoDialog (frmP);                    
        FrmDeleteForm (frmP);

        handled = true;
        break;
    }
    
    return handled;
}

// FUNCTION: MainFormHandleEvent
//
// DESCRIPTION:
//
// This routine is the event handler for the "MainForm" of this 
// application.
//
// PARAMETERS:
//
// eventP
//     a pointer to an EventType structure
//
// RETURNED:
//     true if the event was handled and should not be passed to
//     FrmHandleEvent

static Boolean MainFormHandleEvent(EventType * eventP)
{
    Boolean handled = false;
    FormType * frmP;
    
    char synchronizing[25] = "Synchronizing...";
    char sync_success[25] = "Sync Success!";
    char sync_fail[25] = "Sync Failed.";
    
    char default_url[100] = "http://localhost:8080";
    int rc=0;

    switch (eventP->eType) 
        {
        case menuEvent:
            return MainFormDoCommand(eventP->data.menu.itemID);

        case frmOpenEvent:
            frmP = FrmGetActiveForm();
            MainFormInit(frmP);
            FrmDrawForm(frmP);
            
            //Initialize URL to default:
	   		updateFieldText(GetObjectPtr(MainUrlField), default_url);
	   		
            handled = true;
            break;
            
        case frmUpdateEvent:
            // To do any custom drawing here, first call 
            // FrmDrawForm(), then do your drawing, and 
            // then set handled to true.
            break;
            
        case ctlSelectEvent:
            switch (eventP->data.ctlSelect.controlID) 
            {
            
            case MainSyncButton:   		
		    	//Check to make sure username is filled:
		    		
		   		//Label Status as syncing:
		   		updateFieldText(GetObjectPtr(MainStatusField), synchronizing);
		   				    	
		    	//Start syncing:
		    	syncStartup();
		   		rc = iscEngineSync(hSyncer);
		   				   				    	
		   		//Mark success or failure:	   		
	   			if (rc == ISCRTN_Succeeded) 
					updateFieldText(GetObjectPtr(MainStatusField), sync_success);
				else
					updateFieldText(GetObjectPtr(MainStatusField), sync_fail);

		   		
		   		//Clean up sync handles:
		   		syncShutdown();

		    	break;
			}
			
			break;

        default:
            break;
        }
    
    return handled;
}

int syncStartup() 
{
	ISCPROPERTY properties[] = {{ISCPROP_SyncUser,        NULL},
	                            {ISCPROP_SyncPassword,    NULL},	            
	};  
	int       numProperty = 0;

	char url[100] = {'\0'};
	int       rc = 0;
	
	char targetDir[20] = "";
	
	//allocate 50 bytes for user and password buffer
	char userName[50] = {'\0'};
	char password[50] = {'\0'};
	

	syncShutdown();

	getFieldText(GetObjectPtr(MainUsernameField), userName, 50);
	getFieldText(GetObjectPtr(MainPasswordField), password, 50);
	getFieldText(GetObjectPtr(MainUrlField), url, 100);
	
	
	properties[numProperty++].value = userName;
	properties[numProperty++].value = password;
	
	rc = iscServiceOpenEx(url, properties, numProperty, &hSyncServ);
	if (rc != ISCRTN_Succeeded) 
		return rc;

	rc = iscConfigOpen(hSyncServ, targetDir, &hSyncConf);
	if (rc != ISCRTN_Succeeded) 
		return rc;

	rc = iscEngineOpen(hSyncConf, &hSyncer);
	if (rc != ISCRTN_Succeeded) 
		return rc;

	return rc;
}

void syncShutdown()
{
	
	if (hSyncer != NULL) {
	   iscEngineClose(hSyncer);
       hSyncer = NULL;
	}

	if (hSyncConf != NULL) {
	   iscConfigClose(hSyncConf);
	   hSyncConf = NULL;
	}

	if (hSyncServ != NULL) {
	   iscServiceClose(hSyncServ);
	   hSyncServ = NULL;
	}
}


// FUNCTION: AppHandleEvent
//
// DESCRIPTION: 
//
// This routine loads form resources and set the event handler for
// the form loaded.
//
// PARAMETERS:
//
// event
//     a pointer to an EventType structure
//
// RETURNED:
//     true if the event was handled and should not be passed
//     to a higher level handler.

static Boolean AppHandleEvent(EventType * eventP)
{
    UInt16 formId;
    FormType * frmP;

    if (eventP->eType == frmLoadEvent)
    {
        // Load the form resource.
        formId = eventP->data.frmLoad.formID;
        frmP = FrmInitForm(formId);
        FrmSetActiveForm(frmP);

        // Set the event handler for the form.  The handler of the 
        // currently active form is called by FrmHandleEvent each 
        // time is receives an event.
        switch (formId)
        {
        case MainForm:
            FrmSetEventHandler(frmP, MainFormHandleEvent);
            break;

        default:
            break;

        }
        return true;
    }

    return false;
}

// FUNCTION: AppEventLoop
//
// DESCRIPTION: This routine is the event loop for the application.

static void AppEventLoop(void)
{
    UInt16 error;
    EventType event;

    do {
        // change timeout if you need periodic nilEvents
        EvtGetEvent(&event, evtWaitForever);

        if (! SysHandleEvent(&event))
        {
            if (! MenuHandleEvent(0, &event, &error))
            {
                if (! AppHandleEvent(&event))
                {
                    FrmDispatchEvent(&event);
                }
            }
        }
    } while (event.eType != appStopEvent);
}

// FUNCTION: AppStart
//
// DESCRIPTION:  Get the current application's preferences.
//
// RETURNED:
//     errNone - if nothing went wrong

static Err AppStart(void)
{
    UInt16 prefsSize;

    // Read the saved preferences / saved-state information.
    prefsSize = sizeof(db2eqstartPreferenceType);
    if (PrefGetAppPreferences(
        appFileCreator, appPrefID, &g_prefs, &prefsSize, true) != 
        noPreferenceFound)
    {
        // FIXME: setup g_prefs with default values
    }
    
    return errNone;
}

// FUNCTION: AppStop
//
// DESCRIPTION: Save the current state of the application.

static void AppStop(void)
{
    // Write the saved preferences / saved-state information.  This 
    // data will be saved during a HotSync backup.
    PrefSetAppPreferences(
        appFileCreator, appPrefID, appPrefVersionNum, 
        &g_prefs, sizeof(db2eqstartPreferenceType), true);
        
    // Close all the open forms.
    FrmCloseAllForms();
}

// all code from here to end of file should use no global variables
#pragma warn_a5_access on

// FUNCTION: RomVersionCompatible
//
// DESCRIPTION: 
//
// This routine checks that a ROM version is meet your minimum 
// requirement.
//
// PARAMETERS:
//
// requiredVersion
//     minimum rom version required
//     (see sysFtrNumROMVersion in SystemMgr.h for format)
//
// launchFlags
//     flags that indicate if the application UI is initialized
//     These flags are one of the parameters to your app's PilotMain
//
// RETURNED:
//     error code or zero if ROM version is compatible

static Err RomVersionCompatible(UInt32 requiredVersion, UInt16 launchFlags)
{
    UInt32 romVersion;

    // See if we're on in minimum required version of the ROM or later.
    FtrGet(sysFtrCreator, sysFtrNumROMVersion, &romVersion);
    if (romVersion < requiredVersion)
    {
        if ((launchFlags & 
            (sysAppLaunchFlagNewGlobals | sysAppLaunchFlagUIApp)) ==
            (sysAppLaunchFlagNewGlobals | sysAppLaunchFlagUIApp))
        {
            FrmAlert (RomIncompatibleAlert);

            // Palm OS 1.0 will continuously relaunch this app unless 
            // we switch to another safe one.
            if (romVersion <= kPalmOS10Version)
            {
                AppLaunchWithCommand(
                    sysFileCDefaultApp, 
                    sysAppLaunchCmdNormalLaunch, NULL);
            }
        }

        return sysErrRomIncompatible;
    }

    return errNone;
}

// FUNCTION: db2eqstartPalmMain
//
// DESCRIPTION: This is the main entry point for the application.
//
// PARAMETERS:
//
// cmd
//     word value specifying the launch code. 
//
// cmdPB
//     pointer to a structure that is associated with the launch code
//
// launchFlags
//     word value providing extra information about the launch
//
// RETURNED:
//     Result of launch, errNone if all went OK

static UInt32 db2eqstartPalmMain(
    UInt16 cmd, 
    MemPtr /*cmdPBP*/, 
    UInt16 launchFlags)
{
    Err error;

    error = RomVersionCompatible (ourMinVersion, launchFlags);
    if (error) return (error);

    switch (cmd)
    {
    case sysAppLaunchCmdNormalLaunch:
        error = AppStart();
        if (error) 
            return error;

        // start application by opening the main form
        // and then entering the main event loop
        FrmGotoForm(MainForm);
        AppEventLoop();
        
        AppStop();
        break;

    default:
        break;
    }

    return errNone;
}

// FUNCTION: PilotMain
//
// DESCRIPTION: This is the main entry point for the application.
// 
// PARAMETERS:
//
// cmd
//     word value specifying the launch code. 
//
// cmdPB
//     pointer to a structure that is associated with the launch code
//
// launchFlags
//     word value providing extra information about the launch.
//
// RETURNED:
//     Result of launch, errNone if all went OK

UInt32 PilotMain(UInt16 cmd, MemPtr cmdPBP, UInt16 launchFlags)
{
    return db2eqstartPalmMain(cmd, cmdPBP, launchFlags);
}

//Utility helper function
static void getFieldText(FieldPtr fld, char *textBuf, int bufSize)
{
	MemHandle h = FldGetTextHandle(fld);
	char *text;

	if (h)
	{
		text = (char *) MemHandleLock(h);
		StrNCopy(textBuf, text, bufSize);
		textBuf[bufSize - 1] = 0;
		MemHandleUnlock(h);
	}
	// let Palm do the clean-up of fields
} // getFieldText

static void updateFieldText(FieldPtr fld, char *fieldText)
{
	MemHandle h = MemHandleNew((StrLen(fieldText) + 1) * sizeof(char));
	MemHandle hOld = FldGetTextHandle(fld);
	char *text = (char *) MemHandleLock(h);

	StrCopy(text, fieldText);
	MemHandleUnlock(h);
	FldSetTextHandle(fld, h);
	FldDrawField(fld);
	if (hOld) MemHandleFree(hOld);
} // updateFieldText


// turn a5 warning off to prevent it being set off by C++
// static initializer code generation
#pragma warn_a5_access reset
