/************************************************************************
 *
 * Licensed Materials - Property of IBM
 * Program Name: Header file for IBM DB2Everyplace Sync Client API 
 * (c ) Copyright IBM Corp. 1999-2003. All Rights Reserved.
 *
 * US Government Users Restricted Rights 
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp. Licensed Materials - Property of IBM
 *
 ************************************************************************
 */
/************************************************************************
 *
 * File Name:  isyncore.h
 * Description: Define the data types, constants, error codes and
 *              IBM DB2Everplace Sync Client C-API function prototypes
 *
 ************************************************************************
 */

#ifndef _ISYNCORE_H_
#define _ISYNCORE_H_

#ifdef __cplusplus
extern "C" {
#endif
///////////////////////////// Basic Types //////////////////////////////

typedef void                    isy_VOID;
typedef int                     isy_INT;
typedef unsigned int            isy_UINT;
typedef short                   isy_INT16;  // 2-byte int
typedef unsigned short          isy_UINT16; // 2-byte unsigned int
typedef long                    isy_INT32;  // 4-byte int
typedef unsigned long           isy_UINT32; // 4-byte unsigned int
typedef unsigned long           isy_ULONG;	
typedef unsigned char	        isy_BYTE;   // 1-byte
typedef unsigned short	        isy_WORD;
typedef unsigned long           isy_DWORD;
typedef unsigned char	        isy_BOOL;

#define isy_CONST               const 

#if defined(UNICODE) || defined(_UNICODE)
 	typedef isy_UINT16          isy_TCHAR;
#	define isy_TEXT(str)        L##str
#	define isy_T(str)           (isy_TCHAR*) isy_TEXT(str)
#else
	typedef char                isy_TCHAR;
#	define isy_T(str)           (str)
#endif // UNICODE || _UNICODE

#define DSY_ENTRY
#define DSY_IMPORT
#define DSY_EXPORT

typedef isy_VOID* HISCENG;
typedef isy_VOID* HISCCONF;
typedef isy_VOID* HISCSERV;
typedef isy_INT32 HISCCSR;
typedef isy_INT32 HISCSUBS;
typedef isy_INT32 HISCSUBSSET;
typedef isy_INT  (*PFISCPROC)();

// buffer size required for various preferences
#define ISCLEN_SubsSetID              30
#define ISCLEN_SubsSetName            20
#define ISCLEN_SubsSetSignature       20
#define ISCLEN_SubsName               20
#define ISCLEN_Table                  20
#define ISCLEN_Trace                   2
#define ISCLEN_Timeout                12
#define ISCLEN_PerfPropLen			 128

// _______________________________________________________________________
// ISCLISTENARG        data type of information for the sync listener
//                     is a list of string arguments (argc, argv)
typedef struct {
    isy_INT32       argc;
    isy_TCHAR     **argv;
} ISCLISTENARG;
// __________________________________________________________________________
// ISCLISTENCOLUMN     data type of information for the sync listener
//                     is a table column which contains the column position,
//                     primary key sequence, column type, data size, and the
//                     actual column data
typedef struct {
    isy_INT32       pos;
    isy_INT32       pkseq;
    isy_INT32       type;
    isy_INT32       size;
    isy_BYTE       *data;
	isy_TCHAR      *name;
    isy_UINT32      reserved;
} ISCLISTENCOLUMN;
// __________________________________________________________________________
// ISCLISTENCONFLICT   data type of information for the sync listener
//                     is a table record which contains the table name,
//                     the operation, number of columns, and and list of 
//                     columns
typedef struct {
    isy_TCHAR       table[ISCLEN_Table];
    isy_INT32       op;
    isy_INT32       colc;
    ISCLISTENCOLUMN *colv;
    isy_UINT32      reserved;
} ISCLISTENCONFLICT;
// _______________________________________________________________________
// ISCSTATE          data type of the event state:
typedef struct {
    isy_TCHAR       currSubsSet[ISCLEN_SubsSetName];
    isy_TCHAR       currSubs[ISCLEN_SubsSetName];
    isy_UINT32      subsType;
    isy_INT32       syncProg;
} ISCSTATE;
// _______________________________________________________________________
// ISCEVT            data type of a listener event
typedef struct {
    isy_INT32       code;
    isy_UINT32      type;
    isy_INT32       retry;
    ISCSTATE        state;
    ISCLISTENARG   *info;
} ISCEVT;
// _______________________________________________________________________
// iscEngineListenerPF data type of the sync listener
typedef isy_INT32 (*iscEngineListenerPF)(
    isy_UINT32      listenerData,
    ISCEVT         *event,
    isy_VOID       *pExtraInfo);
// _______________________________________________________________________
// ISCPROPERTY       data type of a property
typedef struct {
    isy_TCHAR      *key;
    isy_TCHAR      *value;
} ISCPROPERTY;

/////////////////////// STATUS CODE & CONSTANTS ////////////////////////

// sync engine preferences
#define ISCPREF_Trace                 1
#define ISCPREF_Timeout               2

#define ISCCONST_TraceON              isy_T("y")
#define ISCCONST_TraceOFF             isy_T("n")
#define ISCCONST_TimeoutNever         isy_T("-1")
#define ISCCONST_TimeoutMinimum       isy_T("0")
#define ISCCONST_True                 1
#define ISCCONST_False                0

#define ISCCONST_OpDelete             1
#define ISCCONST_OpInsert             2
#define ISCCONST_OpUpdate             3

#define ISCPROP_SyncUser              isy_T("isync.user")
#define ISCPROP_SyncPassword          isy_T("isync.password")
#define ISCPROP_SyncEncoding          isy_T("isync.encoding")
#define ISCPROP_SyncMessageSize       isy_T("isync.messagesize")
#define ISCPROP_SyncMessageDump       isy_T("isync.messagedump")
#define ISCPROP_SyncProxy             isy_T("isync.proxy")
#define ISCPROP_SyncTrace             isy_T("isync.trace")
#define ISCPROP_SyncTimeout           isy_T("isync.timeout")
#define ISCPROP_SyncCreateImage       isy_T("isync.createimage")
#define ISCPROP_SyncAllowBackup       isy_T("isync.allowbackup")
#define ISCPROP_SyncRetryInterval     isy_T("isync.retryinterval")
#define ISCPROP_SyncDBUrl             isy_T("isync.db.url")
#define ISCPROP_SyncDBUser            isy_T("isync.db.user")
#define ISCPROP_SyncDBPassword        isy_T("isync.db.password")
#define ISCPROP_TargetDBUrl           isy_T("target.db.url")
#define ISCPROP_TargetDBUser          isy_T("target.db.user")
#define ISCPROP_TargetDBPassword      isy_T("target.db.password")
#define ISCPROP_DB2eWriteThrough	  isy_T("db2e.writethrough")

// SyncClient API return codes
#define ISCRTN_Empty                  2
#define ISCRTN_True                   1
#define ISCRTN_False                  0

#define ISCRTN_Ready                  1
#define ISCRTN_Succeeded              0
#define ISCRTN_Failed                -1
#define ISCRTN_Canceled              -2
#define ISCRTN_NotFound              -3
#define ISCRTN_UnknownID             -4
#define ISCRTN_ValTooLong            -5
#define ISCRTN_ValTruncated          -6
#define ISCRTN_OutOfMemory           -7
#define ISCRTN_ResourceBusy          -8
#define ISCRTN_NotPermitted          -9

// Sync listener return codes
#define ISCRTNCB_Default              0
#define ISCRTNCB_Done                 1
#define ISCRTNCB_ReplyNo              2
#define ISCRTNCB_ReplyYes             3

// Event types
#define ISCEVTTYPE_Info               1
#define ISCEVTTYPE_Conflict           2
#define ISCEVTTYPE_Query              3
#define ISCEVTTYPE_Retry              4
#define ISCEVTTYPE_Error              5
#define ISCEVTTYPE_Fatal              6

// Subscription types
#define ISCSUBSTYPE_Config            100
#define ISCSUBSTYPE_File              101
#define ISCSUBSTYPE_DB2e              102
#define ISCSUBSTYPE_Custom            10000

// Information events
#define ISCEVT_InfGeneral             1000
#define ISCEVT_InfSyncStarted         1001
#define ISCEVT_InfPrepMsg             1002
#define ISCEVT_InfSendMsg             1003
#define ISCEVT_InfWaitMsg             1004
#define ISCEVT_InfApplyMsg            1005
#define ISCEVT_InfCancelingSync       1006
#define ISCEVT_InfSubsSetStarted      1007
#define ISCEVT_InfSyncingSubs         1008
#define ISCEVT_InfSubsSetFailed       1009
#define ISCEVT_InfSubsSetCanceled     1010
#define ISCEVT_InfSubsSetSucceeded    1011
#define ISCEVT_InfSyncSucceeded       1012
#define ISCEVT_InfSyncFailed          1013
#define ISCEVT_InfSyncCanceled        1014
#define ISCEVT_InfSyncProg            1015
#define ISCEVT_InfNoNewChange         1016
#define ISCEVT_InfLoginFailed         1017
#define ISCEVT_InfSyncMsgSent         1018
#define ISCEVT_InfStats               1019
#define ISCEVT_InfDBTimeStats         1020
#define ISCEVT_InfDBRowSent           1021
#define ISCEVT_InfDBRowRecv           1022
#define ISCEVT_InfDBBuildTimeStats    1023

// Conflict events
#define ISCEVT_CftReject              2000

// Error events
#define ISCEVT_ErrOpenAdapter         300
#define ISCEVT_ErrLoadAdapter         301
#define ISCEVT_ErrCloseAdapter        302
#define ISCEVT_ErrAuthenticateKey     306
#define ISCEVT_ErrClientCryptoFailed  307
#define ISCEVT_ErrEncryptNotAvail     308
#define ISCEVT_ErrEncryptLibOpen      309
#define ISCEVT_ErrSubsNotFound        311
#define ISCEVT_ErrSubsNotAvail        312
#define ISCEVT_ErrSubsDefAltered      316

#define ISCEVT_ErrAllocResource       400
#define ISCEVT_ErrConnectData         401
#define ISCEVT_ErrDisconnectData      402
#define ISCEVT_ErrNoData              403
#define ISCEVT_ErrSyncDisabled        417
#define ISCEVT_ErrServerException     418
#define ISCEVT_ErrMessageFormat       412
#define ISCEVT_ErrNotFound            413
#define ISCEVT_ErrEndOfData           414
#define ISCEVT_ErrDataTooLong         415
#define ISCEVT_ErrReadOnly            420
#define ISCEVT_ErrOperation           421
#define ISCEVT_ErrUnauthorized        423
#define ISCEVT_ErrNotAvailable        424
#define ISCEVT_ErrNotSupported        425
#define	ISCEVT_ErrSubsTargetDir	      426 // no absolute paths

#define ISCEVT_ErrCloseNetLib         608
#define ISCEVT_ErrOutOfMemory         610
#define ISCEVT_ErrInternal            698

// Fatal events
#define ISCEVT_FatSyncCfgAbort        303
#define ISCEVT_FatAuthenticateFailed  304
#define ISCEVT_FatIncompVersion       310
#define ISCEVT_FatInvalidSession      313
#define ISCEVT_FatSyncGroup           314
#define ISCEVT_FatRegisterDevice      315
#define ISCEVT_FatCreateImage         317
#define ISCEVT_FatServerForbidden     611
#define ISCEVT_FatServerNotFound      612
#define ISCEVT_FatServerNotAvail      614
#define ISCEVT_FatServer              613
#define ISCEVT_FatServerBusy          616
#define ISCEVT_FatNetOpenConn         600
#define ISCEVT_FatNetConnect          601
#define ISCEVT_FatNetSend             602
#define ISCEVT_FatNetReceive          603
#define ISCEVT_FatNetTimeout          604
#define ISCEVT_FatOpenNetLib          606
#define ISCEVT_FatResolveHost         609
#define ISCEVT_FatProtocolNotSupported 615
#define ISCEVT_FatNetUnknown          699

// These errors have been re-categorized as the fatal errors, 
// and their old error constant macros are deplicated. 
// Please use the new fatal error constant macros.
#define ISCEVT_ErrNetConn             ISCEVT_FatNetConnect
#define ISCEVT_ErrSendRequest         ISCEVT_FatNetSend
#define ISCEVT_ErrRecvReply           ISCEVT_FatNetReceive
#define ISCEVT_ErrRecvTimeout         ISCEVT_FatNetTimeout
#define ISCEVT_ErrRecvAck             605

#define ISCEVT_TryNetConn             ISCEVT_TryNetConnect
#define ISCEVT_TrySendRequest         ISCEVT_TryNetSend
#define ISCEVT_TryRecvReply           ISCEVT_TryNetReceive
#define ISCEVT_TryRecvTimeout         ISCEVT_TryNetTimeout

// Retry events
#define ISCEVT_TryServerBusy          4616
#define ISCEVT_TryNetConnect          4601
#define ISCEVT_TryNetSend             4602
#define ISCEVT_TryNetReceive          4603
#define ISCEVT_TryNetTimeout          4604

// Query events
#define ISCEVT_QueCancel              5000
#define ISCEVT_QueLogin               5001
#define ISCEVT_QueCancelUponError     5002
#define ISCEVT_QueSubsTarget	      5003

////////////////////////// Function Protocols //////////////////////////
// _______________________________________________________________________
// Get general properties
DSY_IMPORT isy_UINT32 iscGetVersion();
DSY_IMPORT isy_INT32 iscGetBuildDate(
    isy_TCHAR           *buildDate,
    isy_CONST isy_INT32  bufLen);
DSY_IMPORT isy_INT32 iscGetCharEncoding(
	isy_TCHAR           *codePage,
	isy_CONST isy_INT32  bufLen);
// _______________________________________________________________________
// Sync Service
DSY_IMPORT isy_INT32 iscServiceOpen(
    isy_CONST isy_TCHAR *host,
    isy_CONST isy_TCHAR *port,
    isy_CONST isy_TCHAR *username,
    isy_CONST isy_TCHAR *password,
    isy_CONST isy_VOID  *reserved,
    HISCSERV            *phSyncServ);
DSY_IMPORT isy_INT32 iscServiceOpenEx(
    isy_CONST isy_TCHAR *protocol,
    ISCPROPERTY         *property,
	isy_INT32            propertyNum,
    HISCSERV            *phSyncServ);
DSY_IMPORT isy_INT32 iscServiceClose(
    HISCSERV             hSyncServ);
// _______________________________________________________________________
// Sync Config
DSY_IMPORT isy_INT32 iscConfigOpen(
    HISCSERV             hSyncServ,
    isy_TCHAR           *path,
    HISCCONF            *phSyncConf);
DSY_IMPORT isy_INT32 iscConfigClose(
    HISCCONF             hSyncConf);
DSY_IMPORT isy_INT32 iscConfigPurge(
    HISCCONF             hSyncConf);
DSY_IMPORT isy_INT32 iscConfigOpenCursor(
    HISCCONF             hSyncConf,
    HISCCSR             *phCursor);
DSY_IMPORT isy_INT32 iscConfigCloseCursor(
    HISCCONF             hSyncConf,
    HISCCSR              hCursor);
DSY_IMPORT isy_INT32 iscConfigGetNextSubsSet(
    HISCCONF             hSyncConf,
    HISCCSR              hCursor,
    isy_TCHAR           *id,
    isy_TCHAR           *name);
DSY_IMPORT isy_INT32 iscConfigEnableSubsSet(
    HISCCONF             hSyncConf,
    isy_TCHAR           *id);
DSY_IMPORT isy_INT32 iscConfigDisableSubsSet(
    HISCCONF             hSyncConf,
    isy_TCHAR           *id);
DSY_IMPORT isy_INT32 iscConfigResetSubsSet(
    HISCCONF             hSyncConf,
    isy_TCHAR           *id);
DSY_IMPORT isy_INT32 iscConfigSubsSetIsEnabled(
    HISCCONF             hSyncConf,
    isy_TCHAR           *id);
DSY_IMPORT isy_INT32 iscConfigSubsSetIsReset(
    HISCCONF             hSyncConf,
    isy_TCHAR           *id);
DSY_IMPORT isy_INT32 iscConfigGetSubsSetStatus(
    HISCCONF             hSyncConf,
    isy_TCHAR           *id);
DSY_IMPORT isy_INT32 iscConfigAbout(
    HISCCONF             hSyncConf,
    isy_TCHAR           *about,
    isy_INT32            aboutLen);
// _______________________________________________________________________
// Sync Engine
DSY_IMPORT isy_INT32 iscEngineOpen(
    HISCCONF             hSyncConf,
	HISCENG             *phSyncer);
DSY_IMPORT isy_INT32 iscEngineClose(
	HISCENG              hSyncer);
DSY_IMPORT isy_INT32 iscEngineGetInfo(
    HISCENG              hSyncer,
	isy_TCHAR           *info,
    isy_INT32            infoLen);
DSY_IMPORT isy_INT32 iscEngineSetListener(
    HISCENG              hSyncer,
    iscEngineListenerPF  syncListener,
    isy_UINT32           syncListenerData);
DSY_IMPORT isy_INT32 iscEngineSetPref(
    HISCENG              hSyncer,
    isy_CONST isy_INT32  prefID,
    isy_CONST isy_TCHAR *prefVal);
DSY_IMPORT isy_INT32 iscEngineGetPref(
    HISCENG              hSyncer,
    isy_CONST isy_INT32  prefID,
    isy_TCHAR           *prefVal,
    isy_CONST isy_INT32  prefLen);
DSY_IMPORT isy_INT32 iscEngineSync(
    HISCENG              hSyncer);
DSY_IMPORT isy_INT32 iscEngineSyncConfig(
    HISCENG              hSyncer);

#ifdef __cplusplus
}
#endif // end of cplusplus

#endif // _ISYNCORE_H_

